if select(2,UnitClass("player")) ~= "MONK" then return end

local LibStub = LibStub
local AceAddon = LibStub("AceAddon-3.0")

local Geodew = AceAddon:GetAddon("Geodew")

local AceDB = LibStub("AceDB-3.0")
local Geodew_CB = AceAddon:NewAddon("Geodew_CB", "AceEvent-3.0","AceTimer-3.0")
local grid
local bar
local SpellID = 123986

function Geodew_CB:OnInitialize()
	self.db = AceDB:New("Geodew_CBDB",Geodew.Module.raid_cd())
	self.grid=Geodew.UI:CreateGrid(self.db)
	grid = self.grid
	self.bar=Geodew.UI:CreateBar(self.db)
	bar = self.bar
	bar:Set(0,1)
	grid:SetTexture(GetSpellTexture(SpellID))
	self.db.RegisterCallback(self, "OnProfileChanged", "OnEnable")
	self.db.RegisterCallback(self, "OnProfileCopied", "OnEnable")
	self.db.RegisterCallback(self, "OnProfileReset", "OnEnable")
end

function Geodew_CB:OnEnable()
	grid:Update()
	bar:Update()
	self:PLAYER_SPECIALIZATION_CHANGED()
	self:RegisterEvent("PLAYER_SPECIALIZATION_CHANGED")
end

local UnitInRaid = UnitInRaid
local UnitInParty = UnitInParty
local GetNumGroupMembers = GetNumGroupMembers
local string_format = string.format
local UnitBuff = UnitBuff
local select = select
local pairs = pairs
local spell_power= Geodew.spell_power
local unit_health_deficit = Geodew.unit_health_deficit
local predict = Geodew.predict
local predict_total = Geodew.predict_total
local min = min
local GetCritChance = GetCritChance
local UnitIsPVP = UnitIsPVP
local GetTime = GetTime
local UnitPosition = UnitPosition
local UnitDistanceSquared = UnitDistanceSquared
local GetSpellCooldown = GetSpellCooldown
local GetPlayerFacing = GetPlayerFacing
local math_abs = math.abs
local math_cos = math.cos
local math_sin = math.sin
local unit_in_40y = Geodew.unit_in_40y

local function det(x1,y1,x2,y2)
	return x1*y2-x2*y1;
end

local function rect_area(ux,uy,x1,y1,x2,y2)
	return math_abs(det(x1-ux,y1-uy,x2-ux,y2-uy))
end

local function unit_in_rectangle(u,x1,y1,x2,y2,x3,y3,x4,y4,a)
	local ux,uy = UnitPosition(u)
	return math_abs(rect_area(ux,uy,x1,y1,x2,y2) + rect_area(ux,uy,x2,y2,x3,y3) + rect_area(ux,uy,x3,y3,x4,y4) + rect_area(ux,uy,x4,y4,x1,y1) - a ) < 1e-8
end

local function get_player_rectangle(radius,length,back)
	local a = radius*2 * (length+back) *2
	local px,py = UnitPosition("player")
	local player_facing = GetPlayerFacing()
	local c,s = math_cos(player_facing),math_sin(player_facing)
	local dx,dy = length*c,length*s
	local bdx,bdy = back*c,back*s
	local x1,y1 = px+radius*s,py-radius*c
	local x2,y2 = px-radius*s,py+radius*c
	return x1-bdx,y1-bdy,x2-bdx,y2-bdy,x2+dx,y2+dy,x1+dx,y1+dy,a
end

local function ready()
	local start, duration = GetSpellCooldown(SpellID)
	local gtime = GetTime()
	return (start == 0 or duration <= 10) or start+duration<= gtime + Geodew_CB.db.profile.raid_cd_prepare_time
end

function Geodew_CB:TimerFeedback()
	if ready() ~= true then
		grid:Hide()
		bar:Hide()
		return
	end
	grid:SetCD(SpellID)
	local format_str
	local members
	if UnitInRaid("player") then
		format_str = "raid"
		members = GetNumGroupMembers()
	elseif UnitInParty("player") then
		format_str = "party"
		members = GetNumGroupMembers()
	else
		format_str="player"
		members = 1
	end
	local factor = self.db.profile.radius_factor
	
	local x1,y1,x2,y2,x3,y3,x4,y4,a = get_player_rectangle(6*factor,40*factor,3*factor)

	local counts = 0
	local i
--[[	local spell = spell_power() * 7.2
	local health_deficits = 0
	local crit = GetCritChance() /100
	local pvp = UnitIsPVP("player")]]
	for i = 1,members,1 do
		local u
		if i == members then
			if format_str == "raid" then
				u = string_format("%s%d",format_str,i)
			else
				u = "player"
			end
		else
			u = string_format("%s%d",format_str,i)
		end
		if unit_in_40y(u) then
			if unit_in_rectangle(u,x1,y1,x2,y2,x3,y3,x4,y4,a) then
				counts = counts + 1
			end
--			health_deficits = health_deficits + predict(u,spell,crit,pvp)
		end
	end
	if counts == 0 then
		grid:Hide()
		bar:Hide()
	else
		if counts < 6 then
			grid:SetHigh()
		elseif counts < 9 then
			grid:SetMid()
		else
			grid:SetLow()
		end
		grid.center_text:SetText(counts)
--		bar:Set(health_deficits,predict_total(spell,counts,crit,pvp))
		grid:Show()
--		bar:Show()
	end
end

function Geodew_CB:PLAYER_SPECIALIZATION_CHANGED()
	local profile = self.db.profile
	if profile.enable == true and GetSpecialization() == 2 and select(5,GetTalentInfo(1,1,1)) == true  then
		if self.timer then
			self:CancelTimer(self.timer)
		end
		bar:Hide()
		self.timer = self:ScheduleRepeatingTimer("TimerFeedback", profile.timer)
	else
		if self.timer then
			self:CancelTimer(self.timer)
			self.timer = nil
		end
		bar:Hide()
		grid:Hide()
	end
end