if select(2,UnitClass("player")) ~= "MONK" then return end

local LibStub = LibStub
local AceAddon = LibStub("AceAddon-3.0")

local Geodew = AceAddon:GetAddon("Geodew")

local AceDB = LibStub("AceDB-3.0")
local Geodew_EF = AceAddon:NewAddon("Geodew_EF", "AceEvent-3.0","AceTimer-3.0")
local grid
local bar
local SpellID = 191837

function Geodew_EF:OnInitialize()
	self.db = AceDB:New("Geodew_EFDB",Geodew.Module.range())
	self.grid=Geodew.UI:CreateGridDirection(self.db)
	grid = self.grid
	self.bar=Geodew.UI:CreateBar(self.db)
	bar = self.bar
	bar:Set(0,1)
	grid:SetTexture(GetSpellTexture(SpellID))
	self.db.RegisterCallback(self, "OnProfileChanged", "OnEnable")
	self.db.RegisterCallback(self, "OnProfileCopied", "OnEnable")
	self.db.RegisterCallback(self, "OnProfileReset", "OnEnable")
end

function Geodew_EF:OnEnable()
	grid:Update()
	bar:Update()
	self:PLAYER_SPECIALIZATION_CHANGED()
	self:RegisterEvent("PLAYER_SPECIALIZATION_CHANGED")
end

local UnitInRaid = UnitInRaid
local UnitInParty = UnitInParty
local GetNumGroupMembers = GetNumGroupMembers
local string_format = string.format
local UnitBuff = UnitBuff
local select = select
local pairs = pairs
local unit_in_range = Geodew.unit_in_range
local spell_power_total= Geodew.spell_power_total
local unit_health_deficit = Geodew.unit_health_deficit
local min = min

local RJW_name = GetSpellInfo(196725)

local UnitPosition = UnitPosition
local UnitIsUnit = UnitIsUnit

local math_exp = math.exp
local math_sqrt = math.sqrt
local c,dx,dy = 0

function Geodew_EF:TimerFeedback()
	grid:SetPlayerChanneling(SpellID)
	local format_str
	local members
	if UnitInRaid("player") then
		format_str = "raid"
		members = GetNumGroupMembers()
	elseif UnitInParty("player") then
		format_str = "party"
		members = GetNumGroupMembers()
	else
		format_str="player"
		members = 1
	end
	local counts = 0
	local injured_counts = 0
	local i
	local spell_radius = 25 * self.db.profile.radius_factor
	local spell_radius_sq = spell_radius * spell_radius
	local spell = spell_power_total() * 3.66
	if UnitBuff("player",RJW_name,nil,"player") then
		spell = spell * 1.2
	end
	local health_deficits = 0
	for i = 1,members,1 do
		local u
		if i == members then
			if format_str == "raid" then
				u = string_format("%s%d",format_str,i)
			else
				u = "player"
			end
		else
			u = string_format("%s%d",format_str,i)
		end
		if unit_in_range(u,spell_radius_sq) == true then
			counts = counts + 1
			local hd = unit_health_deficit(u)
			if 0 < hd then
				injured_counts = injured_counts + 1
				health_deficits = health_deficits + min(hd,spell)
			end
		end
	end
	if counts < 6 then
		grid:SetHigh()
	elseif counts < 19 then
		grid:SetMid()
	else
		grid:SetLow()
	end
	grid.center_text:SetText(counts)
	if 6 < injured_counts then
		health_deficits = health_deficits / injured_counts * 6
	end
	bar:Set(health_deficits,spell*6)
	if counts<c then
		grid:Shift(dx,dy)
		local bottom_text = grid.bottom_text
		bottom_text:SetText(string_format("%.1f",math_sqrt(dx*dx+dy*dy)))
		bottom_text:Show()
	else
		grid:HideDirection()
		grid.bottom_text:Hide()
	end
end

local function optimal_point(raid,np,rsq,sigma)
	local px,py = UnitPosition("player")
	local i
	local op ,ocnt,ox,oy = 0,0,px,py
	local n = #raid
	if n < np then
		np = n
	end
	for i =1,np,2 do
		local xi = raid[i]
		local yi = raid[i+1]
		local j
		for j =1,np,2 do
			if i~=j then
				local xj = raid[j]
				local yj = raid[j+1]
				local ab_x=xj-xi
				local ab_y=yj-yi
				if ab_x*ab_x + ab_y*ab_y <= 4 *rsq then
					local mid_x=(xi+xj)/2
					local mid_y=(yi+yj)/2

					local length=math_sqrt(ab_x*ab_x+ab_y*ab_y)
					ab_x=ab_x/length
					ab_y=ab_y/length
					local temp_ab_y = -ab_x
					ab_x=ab_y
					ab_y=temp_ab_y
					length=math_sqrt(rsq-length*length/4)
					mid_x=mid_x+ab_x*length
					mid_y=mid_y+ab_y*length
					local dtp = (px-mid_x) * (px-mid_x) + (py-mid_y) * (py-mid_y) 
					if dtp <= rsq then
						local cnt = 0
						local k
						for k =1,n,2 do
							local xk = raid[k]-mid_x
							local yk = raid[k+1]-mid_y
							if xk*xk+yk*yk<rsq then
								cnt = cnt + 1
							end
						end
						
						local predict = math_exp(sigma*dtp) * cnt
						--gauss function predict
						
						if cnt < 5 then				--if spell optimal point covers fewer than 6 allies. predict half
							predict = predict /2
						end
						if op < predict then
							op = predict
							ocnt = cnt
							ox = mid_x
							oy = mid_y
						end
					end
				end
			end
		end
	end
	return ocnt+1,ox-px,oy-py
end

local raid = {}

function Geodew_EF:TimerOptimalFeedback()
	local format_str
	local members
	if UnitInRaid("player") then
		format_str = "raid"
		members = GetNumGroupMembers()
	elseif UnitInParty("player") then
		format_str = "party"
		members = GetNumGroupMembers() - 1
	else
		c = 0
		return
	end
	wipe(raid)
	local profile = self.db.profile
	local radius_factor = profile.radius_factor
	local crsq = profile.optimal_range * radius_factor
	crsq = crsq * crsq
	local i
	for i=1,members do
		local u = string_format("%s%d",format_str,i)
		if UnitIsUnit(u,"player") == false and unit_in_range(u,crsq) == true then
			local x,y = UnitPosition(u)
			raid[#raid+1] = x
			raid[#raid+1] = y
		end
	end

	local spell_radius = 25 * radius_factor
	local sigma = -9 / (2 *crsq)
	c,dx,dy = optimal_point(raid,profile.optimal_points*2,spell_radius*spell_radius,sigma)
end

function Geodew_EF:PLAYER_SPECIALIZATION_CHANGED()
	local profile = self.db.profile
	if profile.enable == true  and GetSpecialization() == 2 then
		if self.timer then
			self:CancelTimer(self.timer)
		end
		self.timer = self:ScheduleRepeatingTimer("TimerFeedback", profile.timer)
		if self.timer2 then
			self:CancelTimer(self.timer2)
		end
		c = 0
		if self.db.profile.optimal == true then
			self.timer2 = self:ScheduleRepeatingTimer("TimerOptimalFeedback", profile.optimal_timer)
		end
		bar:Show()
		grid:Show()
	else
		if self.timer then
			self:CancelTimer(self.timer)
			self.timer = nil
			bar:Hide()
			grid:Hide()
		end
		if self.timer2 then
			self:CancelTimer(self.timer2)
			self.timer2 = nil
		end
		c = 0
	end
end