local LibStub = LibStub
local AceAddon = LibStub("AceAddon-3.0")
local Geodew_Options = AceAddon:GetAddon("Geodew_Options")
local Geodew_Options_UI = Geodew_Options.UI
local Geodew_Options_Module = Geodew_Options.Module
local L = LibStub("AceLocale-3.0"):GetLocale("Geodew_Options")

function Geodew_Options_Module:CreateRangeOptions()
	local options = 
	{
		Bar = Geodew_Options_UI:CreateBarOptions(),
		Grid = Geodew_Options_UI:CreateGridOptions(),
		enable =
		{
			name = L["Enable"],
			type = "toggle",
			set = function(info,val)
				local addon = AceAddon:GetAddon(info[1])
				addon.db.profile.enable = val
				addon:PLAYER_SPECIALIZATION_CHANGED()
			end,
			get = function(info)
				local addon = AceAddon:GetAddon(info[1])
				return addon.db.profile.enable
			end,
			width = "full",
		},
		timer = 
		{
			name = L["Timer"],
			type = "range",
			min = 0,
			max = 5,
			step = 0.01,
			set = function(info,val)
				local addon = AceAddon:GetAddon(info[1])
				addon.db.profile.timer = val
				addon:PLAYER_SPECIALIZATION_CHANGED()
			end,
			get = function(info)
				local addon = AceAddon:GetAddon(info[1])
				return addon.db.profile.timer
			end,
		},
		radius_factor = 
		{
			name = L["Radius Factor"],
			type = "range",
			min = 0,
			max = 1,
			set = function(info,val)
				local addon = AceAddon:GetAddon(info[1])
				addon.db.profile.radius_factor = val
			end,
			get = function(info)
				local addon = AceAddon:GetAddon(info[1])
				return addon.db.profile.radius_factor
			end,
		},
		Optimal = 
		{
			name = L["Optimal Predict"],
			desc = L["Warning. The algorithm of predicting the optimal point is extremely slow. O(n^3) Disabling it will greatly improve performance."],
			type = "group",
			args =
			{
				optimal = 
				{
					name = L["Enable"],
					type = "toggle",
					set = function(info,val)
						local addon = AceAddon:GetAddon(info[1])
						addon.db.profile.optimal = val
						addon:PLAYER_SPECIALIZATION_CHANGED()
					end,
					get = function(info)
						local addon = AceAddon:GetAddon(info[1])
						return addon.db.profile.optimal
					end,
					width = "full",
				},
				optimal_points = 
				{
					name = L["Points"],
					type = "range",
					min = 6,
					max = 40,
					step = 1,
					set = function(info,val)
						local addon = AceAddon:GetAddon(info[1])
						addon.db.profile.optimal_points = val
					end,
					get = function(info)
						local addon = AceAddon:GetAddon(info[1])
						return addon.db.profile.optimal_points
					end,
				},
				optimal_timer = 
				{
					name = L["Timer"],
					type = "range",
					min = 0,
					max = 5,
					step = 0.01,
					set = function(info,val)
						local addon = AceAddon:GetAddon(info[1])
						addon.db.profile.optimal_timer = val
						addon:PLAYER_SPECIALIZATION_CHANGED()
					end,
					get = function(info)
						local addon = AceAddon:GetAddon(info[1])
						return addon.db.profile.optimal_timer
					end,
				},
				optimal_range = 
				{
					name = L["Range"],
					type = "range",
					min = 0,
					max = 200,
					set = function(info,val)
						local addon = AceAddon:GetAddon(info[1])
						addon.db.profile.optimal_range = val
					end,
					get = function(info)
						local addon = AceAddon:GetAddon(info[1])
						return addon.db.profile.optimal_range
					end,
				}
			}
		}
	}
	return options
end